import { Frequency, Role } from '@modules/ad/core/domain/ad.types';
import { Candidate } from '../../../types/algorithm.types';
import { Selector } from '../algorithm.abstract';
import { AdReadModel } from '@modules/ad/infrastructure/ad.repository';

export class PassengerOrientedSelector extends Selector {
  select = async (): Promise<Candidate[]> => {
    const queryStringRoles: QueryStringRole[] = [];
    if (this.query.driver)
      queryStringRoles.push({
        query: this.createQueryString(Role.DRIVER),
        role: Role.DRIVER,
      });
    if (this.query.passenger)
      queryStringRoles.push({
        query: this.createQueryString(Role.PASSENGER),
        role: Role.PASSENGER,
      });

    console.log(queryStringRoles);
    return (
      await Promise.all(
        queryStringRoles.map<Promise<AdsRole>>(
          async (queryStringRole: QueryStringRole) =>
            <AdsRole>{
              ads: await this.repository.getCandidates(queryStringRole.query),
              role: queryStringRole.role,
            },
        ),
      )
    )
      .map((adsRole: AdsRole) =>
        adsRole.ads.map(
          (adReadModel: AdReadModel) =>
            <Candidate>{
              ad: {
                id: adReadModel.uuid,
              },
              role: adsRole.role,
            },
        ),
      )
      .flat();
  };

  private createQueryString = (role: Role): string =>
    [
      this.createSelect(role),
      this.createFrom(),
      'WHERE',
      this.createWhere(role),
    ].join(' ');

  private createSelect = (role: Role): string =>
    [
      `SELECT
        ad.uuid,driver,passenger,frequency,public.st_astext(matcher.ad.waypoints) as waypoints,
        "fromDate","toDate",
        "seatsProposed","seatsRequested",
        strict,
        "fwdAzimuth","backAzimuth",
        si.day,si.time,si.margin`,
      role == Role.DRIVER ? this.selectAsDriver() : this.selectAsPassenger(),
    ].join();

  private selectAsDriver = (): string =>
    `${this.query.route?.driverDuration} as duration,${this.query.route?.driverDistance} as distance`;

  private selectAsPassenger = (): string =>
    `"driverDuration" as duration,"driverDistance" as distance`;

  private createFrom = (): string =>
    'FROM ad LEFT JOIN schedule_item si ON ad.uuid = si."adUuid"';

  private createWhere = (role: Role): string =>
    [this.whereRole(role), this.whereStrict(), this.whereDate()].join(' AND ');

  private whereRole = (role: Role): string =>
    role == Role.PASSENGER ? 'driver=True' : 'passenger=True';

  private whereStrict = (): string =>
    this.query.strict
      ? this.query.frequency == Frequency.PUNCTUAL
        ? `frequency='${Frequency.PUNCTUAL}'`
        : `frequency='${Frequency.RECURRENT}'`
      : '';

  private whereDate = (): string => {
    const whereDate = `(
      (
        "fromDate" <= '${this.query.fromDate}' and "fromDate" <= '${this.query.toDate}' and
        "toDate" >= '${this.query.toDate}' and "toDate" >= '${this.query.fromDate}'
      ) OR (
        "fromDate" >= '${this.query.fromDate}' and "fromDate" <= '${this.query.toDate}' and
        "toDate" <= '${this.query.toDate}' and "toDate" >= '${this.query.fromDate}'
      ) OR (
        "fromDate" <= '${this.query.fromDate}' and "fromDate" <= '${this.query.toDate}' and
        "toDate" <= '${this.query.toDate}' and "toDate" >= '${this.query.fromDate}'
      ) OR (
        "fromDate" >= '${this.query.fromDate}' and "fromDate" <= '${this.query.toDate}' and
        "toDate" >= '${this.query.toDate}' and "toDate" >= '${this.query.fromDate}'
      )
    )`;
    return whereDate;
  };
}

export type QueryStringRole = {
  query: string;
  role: Role;
};

type AdsRole = {
  ads: AdReadModel[];
  role: Role;
};
